use strict;
use warnings;
use db_specials;

BEGIN { push(@INC, "../store/"); };

our $verbose = 0;

# Globals for tablenames
my $RECIPE_TABLE_NAME = "recipe";
my $INGREDIENT_TABLE_NAME = "ingredient";
my $RECIPE_INGREDIENT_TABLE_NAME = "recipeingredient";

# Globals for the lengths of items that can be inserted into the 
# database.
my $INGREDIENT_NAME_MAX_LENGTH = 24;
my $QUANTITY_MAX_LENGTH = 24;
my $PREPARATION_MAX_LENGTH = 24;

# Inserts a recipe into the recipe database by parsing the
# ingredients to quantity, perparation, and actual ingredient
# Arguments:
# 				$recipeName: Name of the recipe (string)
# 				$ingredientsRef: Reference to string array of ingredients
#				$directionsRef: Array of directions
# Returns:
# 				Nothing
sub InsertRecipe {

	# Arguments
    my $recipeName = shift;
    my $ingredientsRef = shift;
    my $directionsRef = shift;

    my $recipeId;
    my $rows;
	my $name;

	# Connect to db
    my $dbh = ConnectToDatabase();
	
	# See if the recipe already exists
	$name = GetScalarValue($dbh, $RECIPE_TABLE_NAME, "name", "name = ?", $recipeName);
	
	# Recipe exists so lets get out of here
	if (defined($name)) {
		print STDERR "Recipe $name exists, not adding it any more\n" if ($verbose > 1);
		$dbh->disconnect();
		return -1;
	}
	
	# Otherwise lets insert the name first
    $dbh->do("INSERT INTO $RECIPE_TABLE_NAME (name, directions) VALUES (?, ?)",
	     undef, $recipeName, join(" ", @$directionsRef));

    $recipeId = GetLastInsertedId($dbh);

	# Now go through the ingredients, prune them, and then
	# insert them into the db associated with recipeId
    for my $ingredient (@$ingredientsRef) {
		my $trueIngredient;
		my $trueIngredientId;
		my $quantity;
		my $preparation;
		my $query;
		
		$trueIngredient = $ingredient;
		$quantity = $ingredient;
		$preparation = $ingredient;
		
		# Prune the ingredient (is allrecipes.com ingredient)
		($trueIngredient, $quantity, $preparation) = parseIngredient($ingredient);
		
		# Database freaks out so cut off the length of strings
		# Some of the strings get very very long
		# TODO: Find a better way to prune the strings
		$trueIngredient = substr($trueIngredient, 0, $INGREDIENT_NAME_MAX_LENGTH) if $trueIngredient;
		$quantity         = substr($quantity, 0, $QUANTITY_MAX_LENGTH) if $quantity;
		$preparation    = substr($preparation, 0, $PREPARATION_MAX_LENGTH) if $preparation;
		
		# Lets see if the ingredient already exists or not
		$trueIngredientId = GetScalarValue($dbh, "$INGREDIENT_TABLE_NAME", "id", "name = ?", $trueIngredient);	
		
		# If it does not, insert it and update the trueIngredientId
		if (not defined($trueIngredientId)) {
			# Insert ingredient into the database and get the id
			$rows = $dbh->do("INSERT INTO $INGREDIENT_TABLE_NAME (name) VALUES (?)",
									   undef, $trueIngredient);       

			$trueIngredientId = GetLastInsertedId($dbh);
		}

		# Finally insert the ingredient, quantity, and preparation
		$query = qq{INSERT INTO $RECIPE_INGREDIENT_TABLE_NAME 
							   (recipeId, ingredientId, quantity, preparation)
							   VALUES 
							   (?, ?, ?, ?)};
							   
		$dbh->do($query, undef, $recipeId, $trueIngredientId, 
					   $quantity, $preparation);
    }

	# Alrighty, lets go home, and not forget to wrap up everything
    $dbh->disconnect();
	return 0;
}

# These words are used to prune out the quantity and prepartion infor
# from the allrecipes.com ingredient string
my $MEASURE_WORDS = "can|cans|cup|slices|envelope|slice|cups|teaspoon|teaspoons|package|jigger|fluid ounces|tablespoon|tablespoons|cans|fluid ounce|ounce|ounces|pound|pounds";
my $PREPARATION_WORDS = "warm|frozen cut|diced|boil|cold";

# Parses an ingredient which has the quantity, preparation, and ingredient
# information in it. These are the strings that are returned by allrecipes.com
# Arguments:
#					$falseIngredient: Ingredient returned from allrecipes.com
# Returns:
# 					A tuple of  ($trueIngredient, $quantity, $preparation);
sub parseIngredient {
	my $falseIngredient = shift;
	my $trueIngredient;
	my $quantity;
	my $preparation;
	open(BAD_FILE, ">>bad_files.txt");
	# Go through a set of keywords that match an ingredient
	if ($falseIngredient =~ m/((\d* )?\d+(\/\d+)?( \(.+\))? ($MEASURE_WORDS)) (.*)/io) {
		$quantity = $1;
		$trueIngredient = $6;
		($trueIngredient, $preparation) = parseTrueIngredient($trueIngredient);
	}
	elsif ($falseIngredient =~ m/(\d+( \(.+\))?) (.*)/) {
		$quantity = $1;
		$preparation = "";
		$trueIngredient = $3;
		($trueIngredient, $preparation) = parseTrueIngredient($trueIngredient);
	}
	else {
		$trueIngredient = $falseIngredient;
		$quantity = undef;
		$preparation = undef;
		print BAD_FILE "Don't know how to parse: $falseIngredient\n";
	}
	close(BAD_FILE);
	
	print "\tFalse Ingredient is $falseIngredient\n"  if ($verbose > 2);
	print "\tQuantity is $quantity\n" if ($quantity && $verbose > 2);
	print "\tpreparation is $preparation\n" if ($preparation && $verbose > 2);
	print "\tTrue Ingredient is $trueIngredient\n" if ($verbose > 2);
	return ($trueIngredient, $quantity, $preparation);
}

# A helper method for parseIngredient
# Assumes that the quantity information has been extracted
# from the ingredient of allrecipes.com
# Arguments:
#				$ingredient: Ingredient with quantity information extracted
# Returns 
# 		A tuple ($trueIngredient, $preparation);
# * Splits the ingredient using a ',' and assumes the ending part is the
#    preparation
# * Uses PREPARATION_WORDS to extract words into $preparation
# * Ignores the parenthesis at the end of an ingredient
sub parseTrueIngredient {
	my $trueIngredient = shift;
	my $preparation;
	my $ignoreStrPos;

	($trueIngredient, $preparation) = split(", ", $trueIngredient);
	
	if (not $preparation) {
		if ($trueIngredient =~ /((\w+)?($PREPARATION_WORDS)(w+)?) (.*)/) {
			$trueIngredient = $5;
			$preparation     = $1;
		}
	}
	
	$ignoreStrPos = index( $trueIngredient, "(");

	if ($ignoreStrPos >= 0) {
		$trueIngredient = substr($trueIngredient, 0, $ignoreStrPos - 1);
	}

	return ($trueIngredient, $preparation);
}

# A handy method to test the parsing part
# Arguments:
#		$testFile: The parser_test_cases files which should be in the
#       format shown for "parser_test_cases.txt", ie
# <BEGIN_FORMAT>
# ^AllRecipe String
# ^	quantity
# ^	preparation
# ^	ingredient
# <END_FORMAT>
# '^' sign represents the left boundary of a file as opened in a text editor. (There 
# should be not '^' symbols in the test cases file).
#  Anything after # symbol is ignored (hence comments can be added that way).
# Return:
#			Non negative numbers indicate success while negative numbers failure.
# Will also print any erroneous test cases on stderr.
sub TestParsingIngredients {
	my $testFile = shift;  #"parser_test_cases.txt";
	my $line;
	my $state = 0;
	my $failed = 0;
	my $falseIngredient, my $expectedQuantity, my $expectedPreparation, my $expectedIngredient;
	open(TEST_CASES, "$testFile") or die("Cannot open $testFile, $@\n");
	while (defined($line = <TEST_CASES>)) {
	        chomp($line);
		if ($line eq "") {
			# Ignore blank lines
			;
		}
		else {
			$line =~ s/^\s+//g;
			if ($line =~ /^#/) {
				# Do nothing
				;
			}
			else {
				if ($line =~ /(.*)#/) {
					$line = $1;
				}
				
				if ($state == 0) {
					$falseIngredient = $line;
					$state++;
				}
				elsif ($state == 1) {
					$expectedQuantity = $line;
					$state++;
				}
				elsif ($state == 2) {
					$expectedPreparation = $line;
					$state++;
				}
				elsif ($state == 3) {
					$expectedIngredient = $line;		
					$state = 0;
					print "$falseIngredient\n\t$expectedQuantity\n\t$expectedPreparation\n\t$expectedIngredient\n" if ($verbose > 2);	
					my $actualTrueIngredient, my $actualQuantity, my $actualPreparation, my $lastTestStatus = 0;
					($actualTrueIngredient, $actualQuantity, $actualPreparation) = parseIngredient($falseIngredient);
					
					$lastTestStatus = TestString($falseIngredient, $expectedQuantity, $actualQuantity, $lastTestStatus);
					$lastTestStatus = TestString($falseIngredient, $expectedPreparation, $actualPreparation, $lastTestStatus);
					$lastTestStatus = TestString($falseIngredient, $expectedIngredient, $actualTrueIngredient, $lastTestStatus);
						
					$failed = 1 if ($lastTestStatus < 0);

				}				
			}
		}
	}

	close(TEST_CASES);
	return -1 * $failed;
}

# A helper method to print out the test information to the console
# if $expectedStr does not match the $actualStr.
# Will decrement $lastTestStatus if the strings don't match.
# and only print the $testTitle if the $lastTestStatus is non negative.
sub TestString {
	my $testTitle       = shift;
	my $expectedStr = shift;
	my $actualStr     = shift;
	my $lastTestStatus = shift;
	
	if ($expectedStr ne $actualStr) {
		if ($lastTestStatus >= 0) {
			print STDERR "Test case: ", $testTitle, "\n";
		}
		print STDERR "\tExpected '$expectedStr' got '$actualStr'\n";
		return $lastTestStatus - 1;
	}
	
	return $lastTestStatus;
}	

1;
