#!/usr/local/bin/perl

#########################
# QFC PARSER
# * Used by the Insert Data script.
# * Parses specials from the QFC website, given a zip code.
#########################


package QFCParser;

use strict;
use WWW::Mechanize;
use HTTP::Cookies;
use AbstractParser;

=pod 

=head1 TITLE

QFCParser

=head1 AUTHOR

paramsan

=head1 NAME

Concrete implementation of an AbstractParser class specifically for the QFC store. This 
documentation is for internal use only on how to deal with the QFCParser objects.

=head1 SYNOPSIS

	use QFCParser;
	$qfcParser = QFCParser->new();
	$allSpecials = $qfcParser->GetSpecials($zipCode);

=head1 REQUIRES

Perl5.10.0, WWW::Mechanize, HTTP::Cookies, AbstractParser

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

This class specifically implements the GetSpecials() method of the AbstractParser. As such,
Directly constructing this class is not recommended. Please see L<AbstractParserFactory> on how to 
construct instances of concrete AbstractParser(s).

=head1 CLASS VARIABLES

See L<AbstractParser/"CLASS VARIABLES"> for the different class variables that can also 
be used here.

=cut

our @ISA = qw(AbstractParser);


=pod

=head2 new

=over 4

=item Description

The default constructor for constructing QFCParser object.

=item Arguments 

None

=item Returns

Returns a reference to a new QFCParser object.

=item Usage

C<< $qfcParser = QFCParser->new(); >>

=back

=cut	
#constructor
sub new {
    my $class = shift;
	
    # call the constructor of the parent class, AbstractParser.
    my $self = $class->SUPER::new();
	
	$self->{_storeName} = "QFC";
	
    bless $self, $class;
    return $self;
}

#my $verbose = 0; # Verbosity, to expand or minimize print statements in qfc parsing code.

# Albertsons did not work...
# QFC did...
# Perhaps because I did not use any follow_link() functions
# and used $mech->get() all the time, which stored brand new
# cookies everytime. Hence, I always got the same empty page about installflash no matter what...
=pod

=head2 GetSpecials

=over 4

=item Description

Provides implementation for the abstract method of the AbstractParser class.
See L<AbstractParser/"GetSpecials"> for more information. 

=item Arguments 

B<$zipCode:> Zipcode for which to retrieve the specials for. 

=item Returns

A reference to an array of Specials in the following format,

[ [ItemName, ItemPrice],
  [ItemName, ItemPrice]
  ...
];

=item Usage

C<< $specialRef = $qfcParser->GetSpecials(); >>

=back

=cut	

sub GetSpecials {
	my $self = shift;
	my $userZipCode = shift;
	
	$self->QFC_GetSpecialsForZipCode($userZipCode);

}

=pod

=head1 INTERNAL METHODS

=head2 QFC_GetSpecialsForZipCode

=over 4

=item Description

Actual method that does the whole work of getting the QFC specials for a 
certain zip code. The store that is chosen is the closest one to  a certain 
zip code.

=item Arguments 

B<$zipCode:> Zipcode for which to retrieve the specials for. 

=item Returns

A reference to an array of Specials in the following format,

[ [ItemName, ItemPrice],
  [ItemName, ItemPrice]
  ...
];

This method will call die() with pertinant error message 
if anything goes wrong during the parsing of the QFC webpage. 

=item Usage

C<< $specialRef = $self->QFC_GetSpecialsForZipCode(); >>

=back

=cut	
#
# Note: I tried to modularize this function but that broke it.
# After spending few hours trying to see what the problem is
# I reverted back, and since we are so close to final release
# I will not take any risk of introducing any more bugs.
# Hence this function  will perhaps become more modular 
# post release, however I did introduce Abstract interfaces
# and a factory class.
# -paramsan
#
sub QFC_GetSpecialsForZipCode  {
	my $self  = shift;
    my $userZipCode = shift;

	my $store = "QFC"; # The store name

	###################
	## Navigate to the specials page
	my $tagCheck;
	
    print "Searching under zip code $userZipCode\n" if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL);

	# The start URL with $userZipCode appended as a parameter.
    my $startUrl = "http://services.qfc.com/mapquest/storelocatorresults.aspx?search3=0&search15=0&search17=0&search19=0&search20=0&Text29=0&Text44=0&Text48=0&Text49=0&Text50=0&Text51=0&Text52=0&radius=20&address=&city=&state=&postalCode=$userZipCode";

    # Create a new Mechanize object with *explicit* cookies enabled
    my $mech = WWW::Mechanize->new(autocheck => 1, cookie_jar => HTTP::Cookies->new);
    
	# IE is required, otherwise the javascript checks might fail and we can be redirected to 
	# a browser not supported webpage.
    $mech->agent_alias( 'Windows IE 6' );

	# Fetch the page
    $mech->get($startUrl);

	# Lets do some serious parsing to find the next link
	# This code below actually finds the first store (hence the closest one) and 
	# "follows" the link to weekly specials
    my $stream = HTML::TokeParser->new(\$mech->{content});
    my $tag = $stream->get_tag("tr");

	$self->AssertTagValid($tag, __LINE__);
	
	# Way to write content to files to see if the html is fine or not
    $self->WriteContentToFile($mech, "tmp1.html");

    while (defined($tag) && not $tag->[1]{"class"} eq "alt") {
		$tag = $stream->get_tag("tr");
    }

	$self->AssertTagValid($tag, __LINE__);
	
	# Should have used $self->{_verbose} everywhere but oh well, later
    if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL) {
		print $tag->[1]{"class"}, "\n";
	}
	
    $self->AssertTagValid($stream->get_tag("td"), __LINE__); # distance
    $self->AssertTagValid($stream->get_tag("td"), __LINE__); # qfc
    $self->AssertTagValid($stream->get_tag("td"), __LINE__); # Address

    
    $tag = $stream->get_tag("a");
    
	$self->AssertTagValid($tag, __LINE__);
	
	my $store = "QFC";
    my $address = $stream->get_trimmed_text("/a");
    my $phone   = $stream->get_trimmed_text("/td");

    $stream->get_tag("td");
    my $city    = $stream->get_trimmed_text("/td");

    $stream->get_tag("td");
    my $state   = $stream->get_trimmed_text("/td");

    $stream->get_tag("td");
    my $zipCode = $stream->get_trimmed_text("/td");
    
	if (not ($address && $phone && $state && $city && $zipCode)) {
		die("Unable to get store info: store: '$store'  address: '$address' phone: '$phone' city: '$city' state: '$state' zipCode: '$zipCode'\n");
	}

	# Now get the corresponding Weekly ad link
    $stream->get_tag("td");
    $tag = $stream->get_tag("a");
    my $link = "";

    if ($tag->[1]{"id"} =~ m/WeeklyAd/) {
		$link = $tag->[1]{"href"};
		print "Weekly Ad link found following it", $link, "\n" if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL);
    }
	else {
		die("Unable to find the WeeklyAd link on the page at" . __LINE__);
	}

	# I tried changing customer_Frame to storeLocator here since I thought
	# that was a shortcut. But apparently I miss couple of cookies which are 
	# important for the pageLarge.jsp to retrieve
    if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL) {
		print $link, "\n";
    }

	# Perhaps follow some links now, will buy me some cookies, yea!!!
    my $newLink = $mech->find_link(url => $link);
	
    $mech->follow_link(url => $link);
    $mech->follow_link(url_regex => qr/storeReview/);
    $mech->follow_link(text_regex => qr/HTML version/);
    
	# I guess I can stop here, now I can go back to using get, lovely	
	# First I need to know how many pages are there
    $mech->get("http://qfconline.inserts2online.com/pageLarge.jsp?pageNumber=1&drpStoreID=807");

    undef($stream);
    $stream = HTML::TokeParser->new(\$mech->{content});

    $tag = $stream->get_tag("a");
	
	$self->AssertTagValid($tag, __LINE__);
	
    my $pageCount = 0;
    
	# I get my page count using the "1:2:3:4..." 'a' link tags
    while (defined($tag)) {
		if ($tag->[1]{"class"} eq "pageNumber") {
			$pageCount++;
		}
		$tag = $stream->get_tag("a");
    }

	if ($pageCount == 0) {
		die("PageCount is still 0, no pages found, means a problem..." . __LINE__);
	}
	
	# Apparently the links are repeated on top and bottom, and 
	# the current page does not have a link
    $pageCount = $pageCount/2 + 1;
	
    print "Number of pages = ", $pageCount, "\n" if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL);
    
	# Now lets get some specials
    undef($stream);
    $stream = HTML::TokeParser->new(\$mech->{content});

    my $curPage = 1;
    my @allSpecials = ();
    for ($curPage = 1; $curPage <= $pageCount; $curPage++) {

		$mech->get("http://qfconline.inserts2online.com/pageLarge.jsp?pageNumber=$curPage&drpStoreID=807");	
		undef($stream);
		$stream = HTML::TokeParser->new(\$mech->{content});
		
		$tag = $stream->get_tag("div");
		my $price;
		my $item;
		while (defined($tag)) {		
			if ($tag->[1]{"class"} =~ m/itemName/) {
				$item = $stream->get_trimmed_text("/div");
				$stream->get_tag("div");
				$price = $stream->get_trimmed_text("/div");
				$price =~ s/\xA2/c/;
				$price =~ s/'//g;
				$item  =~ s/'//g;
				
				push(@allSpecials, [($item, $price)]);
			}
			$tag = $stream->get_tag("div");
		}	
    }	
 
    if ($self->{_verbose} >= $AbstractParser::VERBOSITY_ALL) {
		for my $special (@allSpecials) {
			print $special->[0], " : ", $special->[1], "\n";
		}
    }
    
    return \@allSpecials;
}

=pod

=head2 AssertTagValid

=over 4

=item Description

Assert if the tag passed in is NOT undef

=item Arguments 

B<$tag:> The tag to check undef for
B<$atLine:> Set this to __LINE__, there for convenient debugging to know at what line 
the tag was undef (and hence can be helpful for finding errors in parsing).

=item Returns

Nothing

This method will die() with pertinent error message displaying the line number (if passed in) if 
the tag is undefined.

=item Usage

C<< $self->AssertTagValid($mech->get_tag("a"), __LINE__); >>

=back

=cut	
sub AssertTagValid {
	my $self  = shift;
	my $tagReturn = shift;
	my $atLine      = shift;
	if (not defined($tagReturn)) {
		die("Warning undef returned for case at $atLine\n");
	}
}

=pod

=head2 CheckTagValue

=over 4

=item Description

Another helper method to make sure the value of the tag matches what is expected.

=item Arguments 

B<$actualValue:> The actual value of the tag.
B<$expectedTagValue:> The expected value of the tag, can be a regex string
B<$atLine:> The line number where this method is called. Recommended to be __LINE__.

=item Returns

Nothing

This method will die() with pertinent error message displaying the line number 
if $actualValue does not match $expectedTagValue.

=item Usage

For example to assert that the returned text is all numbers:

C<< $self->CheckTagValue($mech->get_trimmed_text("/a"), "\d+", __LINE__); >>

=back

=cut	
sub CheckTagValue {
	my $self  = shift;
	my $actualValue    = shift;
	my $expectedValue = shift;
	my $atLine             = shift;
	
	if ($actualValue !~ /^$expectedValue$/) {
		die("Warning $expectedValue is not equal to $expectedValue at line $atLine\n");
	}
}

=pod

=head2 WriteContentToFile

=over 4

=item Description

A handly method used as a debugging tool to write the html to a file 
that is returned during parsing of the specials at different steps.

=item Arguments 

B<$mech:> Reference to the Mechanized object.
B<$fileName:> The file to output the html content to.

=item Returns

Nothing.

This method will warn() the users if filename cannot be opened.

=item Usage

C<< $self->WriteContentToFile($mech, "weekly_specials.hmlt"); >>

=back

=cut	
sub WriteContentToFile {
    my $self  = shift;
    my $mech = shift;
    my $fileName = shift;
    open(TMP_FILE, ">$fileName") or warn("Cannot open file $fileName, $@");
    if (not $@) {
		print TMP_FILE $mech->content;
    }
    close(TMP_FILE);
}

1;