<?php

/**
 * Data model for user Favorites. Handles adding, deleting, and retrieving favorites/saved lists.
 * 
 * @package Models
 *
 */
class Favorites_Model extends AbstractModel
{
	/**
	 * Retrieves a list of the user's favorite foods.
	 * 
	 * @param int $userId The userid to use.
	 * @return array
	 */
	function getSavedFoods($userId)
	{
		$foods = $this->db->queryColumn('select favoriteFood from usersavedfood where userId = ?', array($userId));
		return $foods;
	}

	/**
	 * Retrieves a list of the user's saved recipes (ids only).
	 * 
	 * @param int $userId The userid to use.
	 * @return array
	 */
	function getSavedRecipeIds($userId)
	{
		$recipeIds = $this->db->queryColumn('
			select recipeId
			from usersavedrecipe
			where userId = ?', array($userId));

		return $recipeIds;
	}

	/**
	 * Retrieves a list of the user's saved recipes (including associative data).
	 * 
	 * @param Recipe_Model $recipeModel The recipe model.
	 * @param int $userId The userid to use.
	 * 
	 * @return array
	 */
	function getSavedRecipes($userId, &$recipeModel)
	{
		$recipeIds = $this->getSavedRecipeIds($userId);

		$result = array();
		foreach ($recipeIds as $recipeId) {
			$result[] = $recipeModel->getRecipeById($recipeId);
		}

		return $result;
	}

	/**
	 * Retrieves a list of the user's saved specials (ids only).
	 * 
	 * @param int $userId The userid to use.
	 * @return array
	 */
	function getSavedSpecialIds($userId)
	{
		$specialIds = $this->db->queryColumn('
			select specialId
			from usersavedspecial
			where userId = ?', array($userId));

		return $specialIds;
	}

	/**
	 * Retrieves a list of the user's saved specials (including associative data).
	 * 
	 * @param Special_Model $specialModel The Special Model.
	 * @param int $userId The userid to use.
	 * @return array
	 */
	function getSavedSpecials($userId, &$specialModel)
	{
		$specialIds = $this->getSavedSpecialIds($userId);

		$result = array();
		foreach ($specialIds as $specialId) {
			$result[] = $specialModel->getSpecialById($specialId);
		}

		return $result;
	}

	/**
	 * Adds a favorite food to the user's profile.
	 * 
	 * @param string $food The name of the food to add.
	 * @param int $userId The userid to use.
	 * 
	 */
	function addFood($userId, $food)
	{
		if (!array_search($food, $this->getSavedFoods($userId))) {
			$this->db->query('insert into usersavedfood (userId, favoriteFood) values(?, ?)', array($userId, $food));
			return $this->db->lastInsertId();
		} else {
			return false;
		}
	}

	/**
	 * Adds a recipe to the user's profile.
	 * 
	 * @param int $recipeId The recipe id to add.
	 * @param int $userId The userid to use.
	 * 
	 */
	function addRecipe($userId, $recipeId)
	{
		if (!array_search($recipeId, $this->getSavedRecipeIds($userId))) {
			$this->db->query('insert into usersavedrecipe (userId, recipeId) values(?, ?)', array($userId, $recipeId));
			return $this->db->lastInsertId();
		} else {
			return false;
		}
	}

	/**
	 * Adds a special to the user's profile.
	 * 
	 * @param int $specialId The special id to add.
	 * @param int $userId The userid to use.
	 * 
	 */
	function addSpecial($userId, $specialId)
	{
		if (!array_search($specialId, $this->getSavedSpecialIds($userId))) {
			$this->db->query('insert into usersavedspecial (userId, specialId) values(?, ?)', array($userId, $specialId));
			return $this->db->lastInsertId();
		} else {
			return false;
		}
	}

	/**
	 * Removes a food from the user's profile.
	 * 
	 * @param string $food The name of the food to remove.
	 * @param int $userId The userid to use.
	 * 
	 */
	function removeFood($userId, $food)
	{
		$this->db->query('delete from usersavedfood where userId = ? and favoriteFood = ?',
			array($userId, $food)
		);
	}

	/**
	 * Removes a recipe from the user's profile.
	 * 
	 * @param int $recipeId The recipe id to remove.
	 * @param int $userId The userid to use.
	 * 
	 */
	function removeRecipe($userId, $recipeId)
	{
		$this->db->query('delete from usersavedrecipe where userId = ? and recipeId = ?',
			array($userId, $recipeId)
		);
	}

	/**
	 * Removes a special from the user's profile.
	 * 
	 * @param int $specialId The special id to remove.
	 * @param int $userId The userid to use.
	 * 
	 */
	function removeSpecial($userId, $specialId)
	{
		$this->db->query('delete from usersavedspecial where userId = ? and specialId = ?',
			array($userId, $specialId)
		);
	}
}

?>